using System;
using System.Collections.Generic;
using CoreLibrary.Datos;
using System.Linq;
using CoreLibrary;
using CoreLibrary.Controlador.Colecciones;
using System.Xml.Serialization;
namespace CoreLibrary.Datos.Modelo
{
	
	/// <summary>
	/// Representa una propiedad.
	/// </summary>
	public class Propiedad : ControladorBase
    {
        #region Metodos Estaticos
        /// <summary>
        /// Carga una nueva instancia de propiedad desde valor controlador.
        /// </summary>
        /// <param name="vep"></param>
        /// <returns></returns>
        public static Propiedad cargarPropiedad(ValoresEP vep)
        {
            if (vep == null)
                return null;
            return GestionDatos.Instance.Propiedades.FirstOrDefault(p => p.Id == vep.IdPropiedad);
        }
        #endregion

        #region Constructor
        public Propiedad() { }
        /// <summary>
        /// Crea una nueva instancia de propiedad.
        /// </summary>
        /// <param name="nombre"></param>
        /// <param name="tipoPrimitivo"></param>
        /// <param name="categoria">La categoria debe existir.</param>
        public Propiedad(string nombre,string tipoPrimitivo,string categoria="")
        {
            if (GestionDatos.Instance.Propiedades.Any(p => p.Equals(nombre)))
                throw new CoreException(CoreException.NamePropiedadReply);
            this.Id = GestionDatos.Instance.devolverProximoIdPropiedad();
            this.Nombre = nombre;
            this.TipoPrimitivo = tipoPrimitivo;
            this.Baja = false;
            GestionDatos.Instance.guardarPropiedad(this);
            this.State = State.New;
            if (!categoria.Equals(string.Empty))
            {
                if (GestionDatos.Instance[categoria] == null)
                    new Categoria(categoria);
                this.aadirCategoria(GestionDatos.Instance[categoria]);

            }
        }
      
        #endregion
        


        #region vars
        private string _tipoPrimitivo;
        #endregion
        #region Propiedades publicas
        [XmlAttribute]
        public string TipoPrimitivo
        {
            get
            {
                return this._tipoPrimitivo;
            }
            set
            {
                _tipoPrimitivo = value;
                this.State = State.Modify;
            }
        }
        [XmlAttribute]
        public string Nombre
        {
            get
            {
                return base.Nombre;
            }
            ///El nuevo nombre no puede existir, en caso de que exista lanza una excepcion.
            set
            {
                if (this.Nombre != null)
                 if (GestionDatos.Instance.Propiedades.Any(p => p.Nombre.Equals(value) && p.Id != this.Id))
                    throw new CoreException(CoreException.NamePropiedadReply);
                base.Nombre = value;
            }

        }
        /// <summary>
        /// Devuelve una lista con todas las categorias a las que pertenece la propiedad.
        /// </summary>
        [XmlIgnore]
        public List<Categoria> Categorias 
        {
            get
            {
               return (from c in GestionDatos.Instance.Categorias
                       join cp in GestionDatos.Instance.CategoriaPropiedad on c.Id equals cp.IdCategoria 
                       where cp.IdPropiedad==this.Id
                       select c).ToList(); ///TODO Hay que probar la consulta a ver si funciona.
            }
        }
        /// <summary>
        /// Devuelve el nombre de la categoria si la propiedad pertenece a dicha categoria.
        /// </summary>
        /// <param name="categoria"></param>
        /// <returns></returns>
        [XmlIgnore]
        public Categoria this[string categoria]
        {
            get
            {
                return this.Categorias.FirstOrDefault(c => c.Nombre.Equals(categoria));
            }
        }
        //Devuelve una lista con todas las traducciones de la propiedad.
         [XmlIgnore]
        public List<TraduccionPropiedad> Traducciones
        {
            get
            {
                return (from tp in GestionDatos.Instance.Traducciones
                        where tp.IdPropiedad == this.Id
                        select tp).ToList();
            }
        }
        /// <summary>
        /// Devuelve una lista con todos los valores de esta propiedad.
        /// </summary>
         [XmlIgnore]
        public List<ValoresEP> Valores
        {
            get
            {
                return (List<ValoresEP>)GestionDatos.Instance.TablaPropiedadesValoresEP[this.Id];
            }
        } 
        
        #endregion

        #region Metodos Publicos
        /// <summary>
        /// Devuelve true si un elemento esta asociado a esta propiedad.
        /// </summary>
        /// <param name="elemento"></param>
        /// <returns></returns>
        public bool asociadoAUnElemento(Elemento elemento)
        {
            if(this.Valores.Any(v=>v.IdElemento==elemento.Id))
                return true;
            return false;
        }


        public int Size
        {
            get
            {
                return base.Size + 16 + this.TipoPrimitivo.Length * 8 ;
            }
        }
        public ValoresEP devolverValorPropiedad(int idElemento)
        {
            return this.Valores.FirstOrDefault(vep => vep.IdElemento == idElemento);
        }
        /// <summary>
        /// Devuelve la traduccion de la propiedad para el codigo de idioma pasado por
        /// parametro. En caso de que no exista devuelve null.
        /// </summary>
        /// <param name="codIdioma"></param>
        /// <returns></returns>
        public string devolverTraduccionPropiedad(string codIdioma)
        {
            Idioma idioma = GestionDatos.Instance.devolverIdiomaPorCod(codIdioma);
           TraduccionPropiedad tp = Traducciones.FirstOrDefault(t=>t.IdIdioma==idioma.Id);
            if(tp!=null)
                return tp.Traduccion;
            return null;
        }
        /// <summary>
        /// Devuelve true si la traduccion de la propiedad pasada por parametro pertenece a la instancia actual.
        /// Este metodo se utiliza para los filtros, en caso de que no se pasen por get, no utilizarlo.
        /// </summary>
        /// <param name="traduccion"></param>
        /// <param name="codIdioma"></param>
        /// <returns></returns>
        public bool comprobarSiLaTraduccionPerteneceAEstaPropiedad(string traduccion, string codIdioma = "es")
        {
            Idioma idioma = GestionDatos.Instance.devolverIdiomaPorCod(codIdioma);
            TraduccionPropiedad tp = Traducciones.FirstOrDefault(t => t.IdIdioma==idioma.Id);
            if (tp != null && tp.Traduccion.Equals(traduccion))
                return true;
            return false;

        }
        public bool comprobarSiPertenceALaCategoria(string categoria)
        {
            if (this[categoria] == null)
                return false;
            return true;
        }
        /// <summary>
        /// Aade una categoria a la propiedad.
        /// </summary>
        /// <param name="categoria"></param>
        public void aadirCategoria(Categoria categoria)
        {

            new CategoriaPropiedad(categoria, this);
         
        }
        /// <summary>
        /// Aade una traduccion para la propiedad, si esta no existe la crea en la base de datos
        /// </summary>
        /// <param name="cod"></param>
        /// <param name="traduccion"></param>
        public void aadirTraduccion(string cod, string traduccion)
        {
            Idioma idioma = GestionDatos.Instance.devolverIdiomaPorCod(cod);
            TraduccionPropiedad tp = this.Traducciones.FirstOrDefault(t => t.IdIdioma == idioma.Id);
            if (tp == null)
            {
                tp = new TraduccionPropiedad();
                tp.Traduccion = traduccion;
                tp.IdPropiedad = this.Id;
                tp.IdIdioma = idioma.Id;
                GestionDatos.Instance.guardarTraduccionPropiedad(tp);
                tp.State = State.New;
            }
            else
            {
     
                tp.Traduccion = traduccion;
                GestionDatos.Instance.modificarTraduccion(tp);
                tp.State = State.Modify;
            }

        }
        public Categoria devolverCategoria(string categoria)
        {
            return this.Categorias.FirstOrDefault(c => c.Nombre.Equals(categoria));
        }
        public void guardarCambios()
        {
            GestionDatos.Instance.modificarPropiedad(this);
        }
        #endregion

    }
}

